'use strict';

var assert = require( 'assert' );
var mqttRegexParser = require( '../' );

describe( 'parse', function() {

    describe( 'when a single level topic', function() {
        it( 'should return the regex escaped form', function() {

            var regex = mqttRegexParser( 'hello' );
            assert( regex.exec( 'hello', 'should match topic' ) );

            assert.equal( '^hello$', regex.source );
            assert.equal( false, regex.ignoreCase );
            assert.equal( false, regex.global );
        } );
    } );

    describe( 'when a multiple level topic', function() {
        it( 'should return the regex escaped form', function() {

            var regex = mqttRegexParser( 'hello/world' );
            assert( regex.exec( 'hello/world', 'should match topic' ) );

            assert.equal( '^hello\\/world$', regex.source );
            assert.equal( false, regex.ignoreCase );
            assert.equal( false, regex.global );
        } );
    } );

    describe( 'when a single + capture', function() {
        it( 'should return the captured data correctly', function() {

            var regex = mqttRegexParser( 'hello/+/world' );

            var match = regex.exec( 'hello/small/world', 'should match topic' );

            assert.equal( match[ 1 ], 'small' );
        } );
    } );

    describe( 'when a single # capture', function() {
        describe( 'and one section', function() {
            it( 'should return the captured data correctly', function() {

                var regex = mqttRegexParser( 'hello/world/#' );

                var match = regex.exec( 'hello/world/my', 'should match topic' );

                assert.equal( match[ 1 ], 'my' );
            } );
        } );

        describe( 'and multiple sections', function() {
            it( 'should return the captured data correctly', function() {

                var regex = mqttRegexParser( 'hello/world/#' );

                var match = regex.exec( 'hello/world/my/old/friend', 'should match topic' );

                assert.equal( match[ 1 ], 'my/old/friend' );
            } );
        } );
    } );

    describe( 'when a multiple captures', function() {
        it( 'should return the captured data correctly', function() {

            var regex = mqttRegexParser( 'hello/+/world/+/land/#' );

            var match = regex.exec( 'hello/small/world/big/land/is/everywhere', 'should match topic' );

            assert.equal( match[ 1 ], 'small' );
            assert.equal( match[ 2 ], 'big' );
            assert.equal( match[ 3 ], 'is/everywhere' );
        } );
    } );

    describe( 'when topic contains escapable characters', function() {
        it( 'should return the regex escaped form', function() {

            var regex = mqttRegexParser( 'hello.world' );
            assert( regex.exec( 'hello.world' ) );

            assert.equal( '^hello\\.world$', regex.source );
            assert.equal( false, regex.ignoreCase );
            assert.equal( false, regex.global );
        } );
    } );

    describe( 'when topic contains numbers', function() {
        it( 'should return the regex escaped form', function() {

            var regex = mqttRegexParser( 'hello123' );
            assert( regex.exec( 'hello123' ) );

            assert.equal( '^hello123$', regex.source );
            assert.equal( false, regex.ignoreCase );
            assert.equal( false, regex.global );
        } );
    } );

    describe( 'when pattern with +', function() {
        it( 'should return the regex escaped form', function() {

            var regex = mqttRegexParser( 'hello/+/my/+' );
            assert( regex.exec( 'hello/dog/my/friend' ), 'should match valid topics' );
            assert( !regex.exec( 'hello/dogmy/friend' ), 'should not bleed' );
            assert( !regex.exec( 'hello/bojack/horseman/my/friend' ), 'should not bleed' );

            assert.equal( '^hello\\/([\\w\\.\\-\\$]+)\\/my\\/([\\w\\.\\-\\$]+)$', regex.source );
            assert.equal( false, regex.ignoreCase );
            assert.equal( false, regex.global );
        } );
    } );

    describe( 'when pattern with #', function() {
        it( 'should return the regex escaped form', function() {

            var regex = mqttRegexParser( 'hello/#' );
            assert( regex.exec( 'hello/dog' ), 'should match valid topics' );
            assert( regex.exec( 'hello/dog/my/friend' ), 'should match valid topics' );
            assert( !regex.exec( 'hellosir' ), 'should not bleed' );

            assert.equal( '^hello\\/([\\w\\.\\-\\$]+(\\/[\\w\\.\\-\\$]+)*)$', regex.source );
            assert.equal( false, regex.ignoreCase );
            assert.equal( false, regex.global );
        } );
    } );

} );
